@@ -47,8 +47,8 @@ def mmread_n_clear(  filename = MMAPFILE ):
     """
     read and clear  filename
     """
-    while not os.path.exists(filename):
-        time.sleep(0.01)
+    if not os.path.exists(filename):
+        return  "xxxxxx","1"
     with open(filename, mode="r+", encoding="utf8") as file_obj:
         with mmap.mmap(file_obj.fileno(), length=0, access=mmap.ACCESS_WRITE, offset=0) as mmap_obj:
             text = mmap_obj.read().decode("utf8")
