@@ -634,8 +634,10 @@ class Stream_Enhancer:
         #scale = zoom
         #get the webcam size
 
-
-        height, width, channels = self.frame.shape
+        if len(self.frame.shape)==3:
+            height, width, channels = self.frame.shape
+        else:
+            height, width = self.frame.shape
         #prepare the crop
         centerX = int(height/2)
         centerY = int(width/2)
