@@ -468,6 +468,9 @@ def display2(videodev, save = False,
         measure = 0 #1.7
         measure_fov = vof # config.CONFIG['vof'] #
 
+        tracker = None
+        tracker2 = None
+        orb = None
 
         # -see the values sent from the webpy
         webframen = "-------" # frame number from web.py()
@@ -585,11 +588,92 @@ def display2(videodev, save = False,
                     cv2.resizeWindow(wname, frame.shape[1], frame.shape[0] )
 
 
+
+
+
+                #-------- i tried all---no help for csrt tracking-------
+                #-------- i tried all---no help for csrt tracking-------
+                #-------- i tried all---no help for csrt tracking-------
+                # frame = cv2.bilateralFilter(frame,5,100,20) # preserve edges
+                #frame = cv2.cvtColor(frame,cv2.COLOR_BGR2GRAY)
+                #(T,frame) = cv2.threshold(frame,  100, 255,
+                #                          cv2.THRESH_BINARY | cv2.THRESH_OTSU)
+                #frame = cv2.blur( frame, (6,6) ) # doesnt help
+
+                # ======================== track before zoom
+                if tracker!=None:
+                    #print("tracking",tracker)
+                    ok,bbox=tracker.update(frame)
+                    if not ok:
+                        continue
+                    bbox = [ round(i*10)/10 for i in bbox]
+                    (x,y,w,h)=[v for v in bbox]
+                    cx,cy = round( 10*(x+w/2))/10, round(10*(y+h/2))/10
+                    #print("tracking",ok,bbox," ->", cx,cy)
+                    with open("tracking1.dat", "a" ) as f:
+                        f.write( f"{webframetime[11:]} {webframen} {cx} {cy}\n" )
+                    # ------------ play on cropping -- may further stabilize
+                    cropped = frame[round(y):round(y+h), round(x):round(x+w)]
+                    resu = np.zeros((640,480))
+                    cropped = cv2.normalize(cropped, resu,0,255,cv2.NORM_MINMAX)
+                    #cropped = cv2.cvtColor(cropped,cv2.COLOR_BGR2GRAY)
+                    #cropped = cv2.blur( cropped, (2,2) )
+                    #(T,cropped) = cv2.threshold(cropped,  100, 255,  cv2.THRESH_BINARY | cv2.THRESH_OTSU)
+                    #cropped = cv2.merge([cropped,cropped,cropped] )
+                    frame[round(y):round(y+h), round(x):round(x+w)] = cropped
+
+
+                    # ---ORB feature matching...
+                    #cropped = frame[round(y):round(y+h), round(x):round(x+w)]
+                    #kp,des = orb.detectAndCompute(cropped,None)
+                    #frame[round(y):round(y+h), round(x):round(x+w)] = cv2.drawKeypoints(cropped,kp,None)
+
+                    #--- ups
+                    #if not( (x<0)or(y<0)or(x+w>=frame.shape[1])or(y+h>=frame.shape[0]) ):
+                    #print("rect")
+
+                    # # - the other part HSV histo ---- HISTO TRACKING
+                    # hsv = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV)
+                    # dst = cv2.calcBackProject([hsv],[0],roi_hist,[0,180],1)
+                    # ret, track_window = cv2.meanShift(dst,track_window, term_crit)
+                    # xh,yh,wh,hh = track_window
+
+                    cv2.rectangle(frame,(round(x),round(y)),
+                                  (round(x+w),round(y+h)),(0,255,0),1,1)
+                    cv2.line(frame,(round(cx),round(cy)),
+                             ( round(cx),round(cy) ) ,(0,255,0),2,1)
+
+                    # cv2.rectangle(frame, (xh,yh), (xh+wh,yh+hh), (0,0,255),1,1)
+                    # cxh,cyh= round( 10*(xh+wh/2))/10, round(10*(yh+hh/2))/10
+                    # cv2.line(frame,(round(cxh),round(cyh)),(round(cxh),round(cyh)),
+                    #          (0,0,255),2,1)
+
+                # ================= track2
+                if tracker2!=None:
+                    #print("tracking",tracker)
+                    # frame = cv2.blur( frame, (4,4) )
+                    ok2,bbox2=tracker2.update(frame)
+                    bbox2 = [ round(i*10)/10 for i in bbox2]
+                    #print("\ntracking2",ok2,bbox2)
+                    (x2,y2,w2,h2)=[v for v in bbox2]
+                    #if not( (x<0)or(y<0)or(x+w>=frame.shape[1])or(y+h>=frame.shape[0]) ):
+                    #print("rect")
+                    cv2.rectangle(frame,(int(x2),int(y2)),(int(x2+w2),int(y2+h2)),(0,255,255),1,1)
+                    cx2,cy2 = round( 10*(x2+w2/2))/10, round(10*(y2+h2/2))/10
+                    cv2.line(frame,(int(cx2),int(cy2)),
+                             ( int(cx2),int(cy2)
+                             ),(0,255,255),2,1)
+                    with open("tracking2.dat", "a" ) as f:
+                        f.write( f"{webframetime} {webframen} {cx2} {cy2}\n" )
+
+
+                #=========================== ZOOM ME and OTHERS =======
+
                 if zoomme>1: # FUNNY - it continues to zoom where the mouse pointer is !!!!
                     if senh.add_frame(frame):
                         # print("avi..")
                         senh.setbox(f"z {zoomme}",  senh.scale)
-                        w,h,c = frame.shape
+                        w,h = frame.shape[0],frame.shape[1]
                         # print(w-centerY1, h-centerX1)
                         senh.zoom( zoomme , int(centerX1-h/2), int(centerY1-w/2)  )
                         frame = senh.get_frame(  )
@@ -619,7 +703,9 @@ def display2(videodev, save = False,
                         senh.setbox(f"JPG",  senh.jpg)
                         frame = senh.get_frame(  )
 
-                # MEASUREMENT
+
+                # MEASUREMENT ==================================
+
                 if measure>0:
                     h,w = frame.shape[0], frame.shape[1]
                     # measure_fov = 110.5 # notebook
@@ -746,6 +832,8 @@ def display2(videodev, save = False,
 
 
 
+                #========================================================
+
                 cv2.imshow(rpi_name, frame ) # 1 window for each RPi
                 if expande >1 :
                     cv2.resizeWindow( rpi_name, int(expande*frame.shape[1]), int(expande*frame.shape[0]) )
@@ -862,6 +950,49 @@ def display2(videodev, save = False,
                     measure = 0
 
 
+                #---------------- trackers--------------
+                if (not frame is None) and (rpi_name!="") and (key == ord('t')):
+                    print("t PRESSED! - track" ,tracker)
+                    tracker = cv2.TrackerCSRT_create() # KCF GOTURN MIL
+                    bbox = cv2.selectROI(frame)
+                    if (len(bbox)<4)or(bbox[-1]<10):
+                        tracker = None
+                        print("i... fail init track")
+                    else:
+                        #bbox = tuple([ i+0.5 for i in bbox ])
+                        ok = tracker.init(frame,bbox)
+
+                        # #------------this is for histotracker
+                        # x,y,w,h = bbox
+                        # track_window = bbox
+                        # # roi = frame[x:x+w, y:y+h]
+                        # roi = frame[ y:y+h, x:x+w]
+                        # hsv_roi =  cv2.cvtColor(roi, cv2.COLOR_BGR2HSV)
+                        # mask = cv2.inRange(hsv_roi, np.array((0., 60.,32.)), np.array((180.,255.,255.)))
+                        # roi_hist = cv2.calcHist([hsv_roi],[0],mask,[180],[0,180])
+                        # cv2.normalize(roi_hist,roi_hist,0,255,cv2.NORM_MINMAX)
+                        # term_crit = ( cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 200, 0.1 ) #iters, eps
+
+                        # orb = cv2.ORB_create() # I dont use now
+
+                if (not frame is None) and (rpi_name!="") and (key == ord('T')):
+                    print("t PRESSED! - track" ,tracker)
+                    tracker2 = cv2.TrackerKCF_create() # KCF GOTURN MIL
+                    bbox2 = cv2.selectROI(frame)
+                    if (len(bbox2)<4) or  (bbox2[-1]<10):
+                        tracker2 = None
+                        print("i... fail init track2")
+                    else:
+                        #bbox2 = tuple([ i+0.5 for i in bbox2 ])
+                        ok2 = tracker2.init(frame,bbox2)
+
+                if (not frame is None) and (rpi_name!="") and (key == ord('u')):
+                    print("u PRESSED! - UNtrack" )
+                    tracker = None
+                    tracker2 = None
+
+
+
 
 if __name__=="__main__":
     Fire( display3)
